# import statements
from time import *

def CRC16(inputlist,length,preload):
	'''This function computes the CCITT CRC as shown in Held 1999 p. 317 --verified OK against Held p. 317   3/12/05 --
	and then takes the ones-complement of the result and reads it backwards -- that is, register 15 becomes LSB  -- to give
	the WJ MPR CRC, as verified against Parkinson examples from the Compact Reader College 3/12/05 '''
	# define some masks for picking off bytes or bits
	lsbmask=0x01
	bytemask=0xff
	register=[]
	#print ' '
	#print 'start of output'
	# fill registers with preload value
	for x in range(0,16,1):
		register=register+[(preload>>x)&lsbmask]
		#print 'register',x,'=',register[x]
	# now extract each byte and bit of input data and run register update
	for x in range(0,length,1):
		inputbyte=inputlist[x]
		#print 'input byte =', hex(inputbyte)
		for y in range(0,8,1):
			#extract the MSB of the input byte
			nextbit=inputbyte>>7  #shift right 7 places
			#print 'bit ',y,'=',nextbit 
			#carry is the XOR of the input bit and the previous value of register 0, the output register
			carry=(register[0]+nextbit)%2   #mod 2 addition same as XOR
			#print 'carry =', carry, '  register[0]=', register[0], '  nextbit=', nextbit
			#print 'carry =', carry
			#for the moment just shift the registers right
			for z in range(0,15,1):   # now we shift registers right -downwards- replicating XOR's w carry where needed
				if (z==3):
					register[z]=(register[z+1]+carry)%2
				elif (z==10):
					register[z]=(register[z+1]+carry)%2
				else:	
					register[z]=register[z+1]
			register[15]=carry   # and put carry into register 15, the input
			#print register
			inputbyte=(inputbyte<<1)&bytemask  #shift byte
	#complement values
	CRCvaluelsb=0
	for z in range(0,16,1):
		register[z]=(register[z]+1)%2
	#print register, ' is the 1s complement'
	for z in range(0,16,1):
		CRCvaluelsb=CRCvaluelsb+(2**(15-z))*register[z]
	#print   'lsb first value=',  hex(CRCvaluelsb)
	return CRCvaluelsb
	#print 'end of output'
	#print ' '
	#print ' '	
#   end of function  CRC16
#
#
def CRC5(inputlist,length,preload):
	'''This function computes the 5-bit CRC as shown in G2 v 1.0.9 annex F. '''
	# define some masks for picking off bytes or bits
	lsbmask=0x01
	bytemask=0xff
	register=[]
	#print ' '
	#print 'start of output'
	# fill registers with preload value
	for x in range(0,5,1):
		register=register+[(preload>>x)&lsbmask]
		print ' in preload register',x,'=',register[x]
	# now extract each byte and bit of input data and run register update
	for x in range(0,length,1):
		inputbyte=inputlist[x]
		print 'input byte =', hex(inputbyte)
		for y in range(0,8,1):
			#extract the MSB of the input byte
			nextbit=inputbyte>>7  #shift right 7 places
			print 'bit ',y,'=',nextbit 
			#carry is the XOR of the input bit and the previous value of register 0, the output register
			carry=(register[0]+nextbit)%2   #mod 2 addition same as XOR
			print 'carry =', carry, '  register[0]=', register[0], '  nextbit=', nextbit
			print 'carry =', carry
			#for the moment just shift the registers right
			for z in range(0,4,1):   # now we shift registers right -downwards- replicating XOR's w carry where needed
				if (z==1):
					register[z]=(register[z+1]+carry)%2
				else:	
					register[z]=register[z+1]
			register[15]=carry   # and put carry into register 15, the input
			#print register
			inputbyte=(inputbyte<<1)&bytemask  #shift byte
	#complement values
	CRCvaluelsb=0
	for z in range(0,5,1):
		register[z]=(register[z]+1)%2
	print register, ' is the 1s complement'
	for z in range(0,5,1):
		CRCvaluelsb=CRCvaluelsb+(2**(4-z))*register[z]
	print   'lsb first value=',  hex(CRCvaluelsb)
	return CRCvaluelsb
	#print 'end of output'
	#print ' '
	#print ' '	
#   end of function  MPR_CRC
#
#
def RN16(seed=0):
	'''This function hashes a non-zero seed value with the CRC16 function to return a new 16-bit
	random number.  If the seed value is zero we use the system time as a seed value.'''
	if seed==0:
		xseed=int(100*time())	# get the system time
	else:
		xseed=int(seed)
		
	byte1=(xseed/256)%256	# use the remainder function to slice off the last 2 bytes
	byte2=(xseed%256)
	inputlist=[byte1,byte2]
	length=2
	preload=0xffff
	RN16value=CRC16(inputlist,length,preload)
	return RN16value

#  end of function RN16

def RNSlot(Q,seed):
	'''This function finds a 16-bit random number and then calculates the divisor
	divided by 2^Q=1.'''
	if Q==0: 
		y=0
	else:
		x=RN16(seed)
		y=x/(2**(16-Q))
	return y
	
	
	

		
		
		
		
